body{font-family:-apple-system,BlinkMacSystemFont,"Segoe UI","Roboto","Oxygen","Ubuntu","Cantarell","Fira Sans","Droid Sans","Helvetica Neue",sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,"Courier New",monospace}*,:after,:before{box-sizing:border-box}button,input,select,textarea,th{font-family:inherit;font-size:inherit;font-weight:inherit}body{font-family:"Roboto","Noto Sans",sans-serif;padding:0;margin:0;font-size:16px;color:#333e48}a{text-decoration:none}a:hover{text-decoration:underline}h1{font-size:28px}p{margin:5px auto}input,select{border:1px solid #ccc;border-radius:3px}input{padding:3px 6px}select{display:inline;width:auto;padding:2px}.Link{background:none!important;border:none;padding:0!important;cursor:pointer;color:#069;text-align:left}.Link:hover{text-decoration:underline}.sLink{background:none!important;border:none;padding:0!important;cursor:pointer;color:#069;vertical-align:-20%;font-size:inherit;font-weight:inherit}.sLink:hover{text-decoration:underline}input::-webkit-input-placeholder,textarea::-webkit-input-placeholder{color:#ccc}input:-ms-input-placeholder,textarea:-ms-input-placeholder{color:#ccc}input::-ms-input-placeholder,textarea::-ms-input-placeholder{color:#ccc}input::placeholder,textarea::placeholder{color:#ccc}#body{display:flex;flex-direction:column;max-height:100vh;padding:10px 0 0}#body>*{flex:0 0 auto}#body>.MainSpace{flex:0 1 auto;overflow-y:hidden}.MainSpace{display:flex;flex-flow:row;padding:0;max-width:100vw;justify-content:space-between}.MainSpace>*{flex:0 0 auto}.MainSpace>.Results{margin:10px;flex:0 1 auto;overflow:hidden}.MainSpace>.Results>.Scroller{max-width:100%;max-height:100%;overflow:auto}.MainSpace>.FieldsList{margin:10px 0 10px 10px}.ModelSelector{margin:0 auto;text-align:center;text-align-last:center;border:none;padding:1px;font-size:28px;color:#333e48}.ModelSelector:hover{border:1px solid #ccc;border-radius:3px;padding:0}.Filters{margin:0 auto;display:flex;align-items:center;flex-direction:column}.Filters p{margin:5px}input.FilterValue,select.FilterValue,select.Lookup{width:100%}.Filters table,.Filters table.Flat td,.Filters table.Flat th{border:none;padding:3px 1px}.FieldsList{margin:0;overflow-y:auto}.FieldsList table{border:1px solid #ccc;border-radius:3px}.FieldsList tbody,.FieldsList td,.FieldsList th{border:none;vertical-align:top}.FieldsList td,.FieldsList th{padding:1px}.FieldsList td{text-align:left}.Results{position:relative}.Results .Freeze{position:-webkit-sticky;position:sticky;top:2px}.Results th{background-color:#eee}.Results .Scroller,.Results td,.Results th{border:1px solid #ccc;border-radius:2px}.Results tr{line-height:120%}.Results td,.Results th{padding:5px}.Results td.number,.Results td.time{text-align:right}.Results td.HoriBorder,.Results th.HoriBorder{border-top:2px solid #000}.Results td.LeftBorder,.Results th.LeftBorder{border-left:2px solid #000}.Results td.Empty{border:none}.Results th{text-align:center}a.Logo{position:absolute;margin:0 15px;font-size:40px;font-family:"Anton",sans-serif;cursor:pointer;color:#333e48}a.Logo>span{display:inline-block}a.Logo>span.Version{font-size:20px}div.Index{margin:30px}div.Index>div{width:46%;float:left;color:#333e48;border:1px solid #ccc;border-radius:3px;margin:30px 2% 0;padding:0 2% 2%}div.Index .Link{display:inline-block;font-size:20px}.EditSavedView{margin:50px auto 30px}.EditSavedView .SavedViewTitle{font-weight:700;font-size:28px}.EditSavedView form{border:1px solid #ccc;border-radius:3px;padding:15px 15px 12px}.EditSavedView .SavedViewName{font-size:20px;width:100%}.EditSavedView textarea{border:1px solid #ccc;border-radius:3px;padding:10px;min-width:500px;width:100%;min-height:100px}.EditSavedView th{padding:2px;font-weight:700;text-align:left}.EditSavedView td{padding:2px}.EditSavedView table{padding:5px 2px}.SavedViewActions{padding:10px 20px;display:flex;justify-content:space-between;align-items:flex-end}td.SavedViewLimit{padding:0}input.RowLimit{padding:1px 3px;width:100px}.CopyToClipboard{font-size:13px}.Overlay{position:absolute;top:0;right:0;left:0;bottom:0;display:flex;justify-content:center;align-items:center;z-index:100;pointer-events:none}.Fade{opacity:.3}
/*# sourceMappingURL=main.d9a60230.chunk.css.map */